/*
 * #%L
 * ToPIA :: Persistence
 * $Id:$
 * $HeadURL:$
 * %%
 * Copyright (C) 2004 - 2012 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.topiatest;

import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.nuiton.i18n.I18n;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.persistence.EntityVisitor;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.TopiaEntityAbstract;

public abstract class NaturalizedEntityAbstract extends TopiaEntityAbstract implements NaturalizedEntity {

    /**
     * Nom de l'attribut en BD : naturalIdNotNull
     */
    protected Integer naturalIdNotNull;

    /**
     * Nom de l'attribut en BD : naturalIdNull
     */
    protected String naturalIdNull;

    private static final long serialVersionUID = 7364006844154918197L;

    static {
    
        I18n.n_("topia.test.common.naturalizedEntity");
        I18n.n_("topia.test.common.naturalIdNotNull");
        I18n.n_("topia.test.common.naturalIdNull");
    }

    @Override
    public void accept(EntityVisitor visitor) throws TopiaException {
        visitor.start(this);
        visitor.visit(this, PROPERTY_NATURAL_ID_NOT_NULL, Integer.class, naturalIdNotNull);
        visitor.visit(this, PROPERTY_NATURAL_ID_NULL, String.class, naturalIdNull);
        visitor.end(this);
    }

    @Override
    public void setNaturalIdNotNull(Integer naturalIdNotNull) {
        Integer oldValue = this.naturalIdNotNull;
        fireOnPreWrite(PROPERTY_NATURAL_ID_NOT_NULL, oldValue, naturalIdNotNull);
        this.naturalIdNotNull = naturalIdNotNull;
        fireOnPostWrite(PROPERTY_NATURAL_ID_NOT_NULL, oldValue, naturalIdNotNull);
    }

    @Override
    public Integer getNaturalIdNotNull() {
        fireOnPreRead(PROPERTY_NATURAL_ID_NOT_NULL, naturalIdNotNull);
        Integer result = this.naturalIdNotNull;
        fireOnPostRead(PROPERTY_NATURAL_ID_NOT_NULL, naturalIdNotNull);
        return result;
    }

    @Override
    public void setNaturalIdNull(String naturalIdNull) {
        String oldValue = this.naturalIdNull;
        fireOnPreWrite(PROPERTY_NATURAL_ID_NULL, oldValue, naturalIdNull);
        this.naturalIdNull = naturalIdNull;
        fireOnPostWrite(PROPERTY_NATURAL_ID_NULL, oldValue, naturalIdNull);
    }

    @Override
    public String getNaturalIdNull() {
        fireOnPreRead(PROPERTY_NATURAL_ID_NULL, naturalIdNull);
        String result = this.naturalIdNull;
        fireOnPostRead(PROPERTY_NATURAL_ID_NULL, naturalIdNull);
        return result;
    }

    @Override
    public List<TopiaEntity> getAggregate() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            result.add(entity);
            result.addAll(entity.getAggregate());
        }
        return result;
    }

    @Override
    public List<TopiaEntity> getComposite() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            if (entity != null) {
                result.add(entity);
                result.addAll(entity.getComposite());
            }
        }
        return result;
    }

    @Override
    public String toString() {
        String result = new ToStringBuilder(this).
            append(PROPERTY_NATURAL_ID_NOT_NULL, this.naturalIdNotNull).
            append(PROPERTY_NATURAL_ID_NULL, this.naturalIdNull).
         toString();
        return result;
    }

} //NaturalizedEntityAbstract
